#!/bin/python
""" surl - a URL shortening application, supports various sites """

# Copyright (c) 2009 Savvas Radevic <vicedar@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import re
import getopt
import urllib
import urllib2

#services.py
try:
    import services
except ImportError:
    import surl.services as services

class _arguments:
    """ surl command line argument parsing and error detection """
    def __init__(self):
        self.arguments = {
            'api-key'   : '',
            'mode'      : 'stdin',
            'in-place'  : False,
            'links-only': False,
            'output'    : '',
            'password'  : '',
            'quiet'     : False,
            'service'   : 'tinyurl.com',
            'username'  : '',
        }

    def usage(self, error = 1):
        """ Prints surl command usage """
        service_list = services.supportedServices().keys()
        service_list.sort()
        print("Purpose: a URL shortening application, supports various sites")
        print("Modes: stdin or -f (default: stdin)")
        print("Usage: " + __file__ + " [OPTION]\n")
        print("  -h, --help\t\t\tThis help message")
        print("  -a, --api-key <key>\t\tUse an API key")
        print("  -f, --file <filename>\t\tEdit the file directly")
        print("  -i, --in-place\t\tSave changes to file directly (Requires: -f)")
        print("  -l, --links-only\t\tOutput only a list of links (old_url => shortened_url)")
        print("  -o, --output <filename>\tOutput to file")
        print("  -p, --password <password>\tYour password")
        print("  -q, --quiet\t\t\tSuppress any extra messages.")
        print("  -u, --username <username>\tYour username\n")
        print("  -s, --service <service>\tThe URL shortening service to use (Default: tinyurl.com) - supported:")
        print("      \t\t\t\t%s" % ', '.join(service_list))
        print("Example:\n  " + __file__ + " -q\n")
        sys.exit(error)

    def parseArgs(self):
        """ Reads the arguments and checks if used properly """
        if sys.argv[0] == __file__:
            sys_args = sys.argv[1:]
        else:
            sys_args = sys.argv

        try:
            opts, args = getopt.getopt(
                sys_args,
                'ha:f:ilo:p:q:s:u:',
                [   'help',
                    'api-key',
                    'file',
                    'in-place',
                    'links-only',
                    'output',
                    'password',
                    'service',
                    'username',
                    'quiet']
            )
        except getopt.GetoptError, e:
            self.usage("ERROR: " + e.msg)

        for opt, arg in opts:
            if opt in ('-h', '--help'):
                self.usage(0)

            if opt in ('-f', '--file'):
                if not arg:
                    self.usage()
                self.arguments['mode'] = 'file'
                self.arguments['file'] = arg
            else:
                self.arguments['mode'] = 'stdin'

            if opt in ('-a', '--api-key'):
                if not arg:
                    self.usage("ERROR: -a <api-key>")
                self.arguments['api-key'] = arg
            elif opt in ('-i', '--in-place'):
                if not self.arguments['mode'] == 'file':
                    self.usage("ERROR: -i should be used with -f <filename>")
                self.arguments['in-place'] = True
            elif opt in ('-l', '--links-only'):
                self.arguments['links-only'] = True
            elif opt in ('-o', '--output'):
                if not arg:
                    self.usage("ERROR: -o <filename>")
                self.arguments['output'] = arg
            elif opt in ('-p', '--password'):
                if not arg:
                    self.usage("ERROR: -p <password>")
                self.arguments['password'] = arg
            elif opt in ('-q', '--quiet'):
                self.arguments['quiet'] = True
            elif opt in ('-s', '--service'):
                if not arg:
                    self.usage("ERROR: -s <servicename>")
                self.arguments['service'] = arg
            elif opt in ('-u', '--username'):
                if not arg:
                    self.usage("ERROR: -u <username>")
                self.arguments['username'] = arg
        return self.arguments

class surl:
    def getInput(self, args):
        """ Read input from stdin or file """
        if args['mode'] == 'stdin':
            #True = not piped, just terminal
            if sys.stdin.isatty():
                _arguments().usage()
            #Get sys.stdin.readlines()
            w_input = sys.stdin.read()
        else:
            #Get file contents
            try:
                w_input = open(args['file']).read()
            except IOError, e:
                sys.exit(e)
        return w_input

    def getLinksListFromInput(self, input):
        """ Return a list of detected links from input """
        url_list = re.findall(self.urlRegexString(), input)
        return url_list

    def getShortenedLinksWithoutInput(self, url_list):
        """ Reads array url_list and returns an array with tuple items:
            (original_link, shortened_link)
        """
        result_list = list()
        for url in url_list:
            result_list.append((url, self.service.get(self.args, url)))
            #item: (original_link, shortened_link)
        return result_list

    def getShortenedLinksWithInput(self, input):
        """ Returns a text string of the input with in-place changed
            shortened links.
        """
        result = re.sub(self.urlRegexString(), self.processRegexMatch, input)
        return result
    def processRegexMatch(self, match):
        """ Gets links for getShortenedLinksWithInput """
        return self.service.get(self.args, match.group(0))

    def getShortenedLink(self, service_link, url, values = dict()):
        """ Return the shortened link
            If values is an empty dictionary, it will use GET instead of POST.
            Arguments:
                * service_link (e.g. example.com/shorten.php)
                * url (the original long_url)
                * values (variables and values passed to the service link,
                  encoded using urllib.encode())
                  Default: empty dictionary {}
        """
        if values:
            #Use POST
            post_fields = urllib.urlencode(values)
            request = urllib2.Request(service_link, post_fields, self.headers())
        else:
            #Use GET
            request = urllib2.Request(service_link, None, self.headers())
        try:
            contents = urllib2.urlopen(request).read()
        except urllib2.HTTPError, e:
            print(str(e) + " " + url + " -- " + e.read() + " ")
            contents = url
        contents = contents.rstrip()
        if not contents:
            return url
        return contents

    def headers(self):
        """ Return headers to be used when contacting services """
        headers = {
            'User-agent': 'surl/0.1',
        }
        return headers

    def runService(self, args):
        """ Main function, requires arguments """
        dictcmd = services.supportedServices()
        #input from file or stdin
        self.input = self.getInput(args)
        #setup service
        try:
            self.service = dictcmd[args['service']]
        except KeyError, e:
            _arguments().usage("ERROR: No such service: " + str(e))
        #setup args
        self.args = args

        #FIXME: Should be controlled with -o
        self.controlOutputStyle()

    def controlOutputStyle(self):
        """ Controls output style (--links-only or not) """
        result = ''
        if self.args['links-only']:
            url_list = self.getLinksListFromInput(self.input)
            list = self.getShortenedLinksWithoutInput(url_list)
            for long_url, short_url in list:
                # long.example.com/of/a/url.html => short.com
                result = result + long_url + ' => ' + short_url + "\n"
        else:
            #full text output with shortened links
            result = self.getShortenedLinksWithInput(self.input)
            if self.args['mode'] == 'stdin':
                result = result.rstrip()
        self.controlOutputMethod(result)

    def controlOutputMethod(self, result):
        """ Controls which output method to use (e.g. -o <file>) """
        print result

    def urlRegexString(self):
        """ Returns the regular expressions string used for catching URLs """
        urlregex = {
            'protocol': '(?:(?:https?|ftp)://)?',
            'byip'    : '\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}',
            'bydomain': '\S+\.(?:com|org|net|gov|mil|biz|info|mobi|name|aero|jobs|museum|travel|[a-z]{2})',
            'rest2'   : '(?:/\S*|/?(?=\s))',
        }
        urlregex_string = urlregex['protocol'] + \
            '(?:' + \
                urlregex['byip'] + '|' + urlregex['bydomain'] + \
            ')' + \
            urlregex['rest2']
        return urlregex_string

def main():
    args = _arguments().parseArgs()
    surl().runService(args)

if __name__ == '__main__':
    main()
